package javax.swing.text;

import java.awt.Container;
import java.awt.Rectangle;
import java.awt.Shape;
import javax.swing.event.DocumentEvent;
import javax.swing.text.html.HTML;
import java.util.BitSet;
import java.util.Vector;
import javax.swing.SizeRequirements;

public abstract class TableView extends BoxView
{
    int[] columnSpans;
    int[] columnOffsets;
    SizeRequirements[] columnRequirements;
    Vector<TableView.TableRow> rows;
    boolean gridValid;
    private static final BitSet EMPTY;
    
    public TableView(Element element) {
        super(element, 1);
        this.rows = new Vector();
        this.gridValid = false;
    }
    
    protected TableView.TableRow createTableRow(Element element) {
        return new TableView.TableRow(element);
    }
    
    @Deprecated
    protected TableView.TableCell createTableCell(Element element) {
        return new TableView.TableCell(element);
    }
    
    int getColumnCount() {
        return this.columnSpans.length;
    }
    
    int getColumnSpan(int n) {
        return this.columnSpans[n];
    }
    
    int getRowCount() {
        return this.rows.size();
    }
    
    int getRowSpan(int n) {
        TableView.TableRow row = this.getRow(n);
        if (row != null) {
            return (int)row.getPreferredSpan(1);
        }
        return 0;
    }
    
    TableView.TableRow getRow(int index) {
        if (index < this.rows.size()) {
            return (TableView.TableRow)this.rows.elementAt(index);
        }
        return null;
    }
    
    int getColumnsOccupied(View view) {
        String s = (String)view.getElement().getAttributes().getAttribute(HTML.Attribute.COLSPAN);
        if (s != null) {
            try {
                return Integer.parseInt(s);
            }
            catch (NumberFormatException ex) {}
        }
        return 1;
    }
    
    int getRowsOccupied(View view) {
        String s = (String)view.getElement().getAttributes().getAttribute(HTML.Attribute.ROWSPAN);
        if (s != null) {
            try {
                return Integer.parseInt(s);
            }
            catch (NumberFormatException ex) {}
        }
        return 1;
    }
    
    void invalidateGrid() {
        this.gridValid = false;
    }
    
    @Override
    protected void forwardUpdate(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        super.forwardUpdate(elementChange, documentEvent, shape, viewFactory);
        if (shape != null) {
            Container container = this.getContainer();
            if (container != null) {
                Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
                container.repaint(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
        }
    }
    
    @Override
    public void replace(int n, int n2, View[] array) {
        super.replace(n, n2, array);
        this.invalidateGrid();
    }
    
    void updateGrid() {
        if (!this.gridValid) {
            this.rows.removeAllElements();
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                View view = this.getView(i);
                if ((view instanceof TableView.TableRow)) {
                    this.rows.addElement((TableView.TableRow)view);
                    TableView.TableRow tableRow = (TableView.TableRow)view;
                    tableRow.clearFilledColumns();
                    tableRow.setRow(i);
                }
            }
            int max = 0;
            int k;
            for (int size = this.rows.size(), j = 0; j < size; max = Math.max(max, k), ++j) {
                TableView.TableRow row = this.getRow(j);
                k = 0;
                for (int l = 0; l < row.getViewCount(); ++l, ++k) {
                    View view2 = row.getView(l);
                    while (row.isFilled(k)) {
                        ++k;
                    }
                    int rowsOccupied = this.getRowsOccupied(view2);
                    int columnsOccupied = this.getColumnsOccupied(view2);
                    if (columnsOccupied > 1 || rowsOccupied > 1) {
                        int n = j + rowsOccupied;
                        int n2 = k + columnsOccupied;
                        for (; j < n; ++j) {
                            for (; k < n2; ++k) {
                                if (j != j || k != k) {
                                    this.addFill(j, k);
                                }
                            }
                        }
                        if (columnsOccupied > 1) {
                            k += columnsOccupied - 1;
                        }
                    }
                }
            }
            this.columnSpans = new int[max];
            this.columnOffsets = new int[max];
            this.columnRequirements = new SizeRequirements[max];
            for (int n3 = 0; n3 < max; ++n3) {
                this.columnRequirements[n3] = new SizeRequirements();
            }
            this.gridValid = true;
        }
    }
    
    void addFill(int n, int n2) {
        TableView.TableRow row = this.getRow(n);
        if (row != null) {
            row.fillColumn(n2);
        }
    }
    
    protected void layoutColumns(int allocated, int[] offsets, int[] spans, SizeRequirements[] children) {
        SizeRequirements.calculateTiledPositions(allocated, null, children, offsets, spans);
    }
    
    @Override
    protected void layoutMinorAxis(int n, int n2, int[] array, int[] array2) {
        this.updateGrid();
        for (int rowCount = this.getRowCount(), i = 0; i < rowCount; ++i) {
            this.getRow(i).layoutChanged(n2);
        }
        this.layoutColumns(n, this.columnOffsets, this.columnSpans, this.columnRequirements);
        super.layoutMinorAxis(n, n2, array, array2);
    }
    
    @Override
    protected SizeRequirements calculateMinorAxisRequirements(int n, SizeRequirements sizeRequirements) {
        this.updateGrid();
        this.calculateColumnRequirements(n);
        if (sizeRequirements == null) {
            sizeRequirements = new SizeRequirements();
        }
        long n2 = 0L;
        long n3 = 0L;
        long n4 = 0L;
        for (SizeRequirements sizeRequirements2 : this.columnRequirements) {
            n2 += (long)sizeRequirements2.minimum;
            n3 += (long)sizeRequirements2.preferred;
            n4 += (long)sizeRequirements2.maximum;
        }
        sizeRequirements.minimum = (int)n2;
        sizeRequirements.preferred = (int)n3;
        sizeRequirements.maximum = (int)n4;
        sizeRequirements.alignment = 0.0f;
        return sizeRequirements;
    }
    
    void calculateColumnRequirements(int n) {
        boolean n2 = false;
        int rowCount = this.getRowCount();
        for (int i = 0; i < rowCount; ++i) {
            TableView.TableRow row = this.getRow(i);
            for (int n3 = 0, viewCount = row.getViewCount(), j = 0; j < viewCount; ++j, ++n3) {
                View view = row.getView(j);
                while (row.isFilled(n3)) {
                    ++n3;
                }
                this.getRowsOccupied(view);
                int columnsOccupied = this.getColumnsOccupied(view);
                if (columnsOccupied == 1) {
                    this.checkSingleColumnCell(n, n3, view);
                }
                else {
                    n2 = true;
                    n3 += columnsOccupied - 1;
                }
            }
        }
        if (!(n2 == 0)) {
            for (int k = 0; k < rowCount; ++k) {
                TableView.TableRow row2 = this.getRow(k);
                for (int n4 = 0, viewCount2 = row2.getViewCount(), l = 0; l < viewCount2; ++l, ++n4) {
                    View view2 = row2.getView(l);
                    while (row2.isFilled(n4)) {
                        ++n4;
                    }
                    int columnsOccupied2 = this.getColumnsOccupied(view2);
                    if (columnsOccupied2 > 1) {
                        this.checkMultiColumnCell(n, n4, columnsOccupied2, view2);
                        n4 += columnsOccupied2 - 1;
                    }
                }
            }
        }
    }
    
    void checkSingleColumnCell(int n, int n2, View view) {
        SizeRequirements sizeRequirements = this.columnRequirements[n2];
        sizeRequirements.minimum = Math.max((int)view.getMinimumSpan(n), sizeRequirements.minimum);
        sizeRequirements.preferred = Math.max((int)view.getPreferredSpan(n), sizeRequirements.preferred);
        sizeRequirements.maximum = Math.max((int)view.getMaximumSpan(n), sizeRequirements.maximum);
    }
    
    void checkMultiColumnCell(int n, int n2, int n3, View view) {
        long n4 = 0L;
        long n5 = 0L;
        long n6 = 0L;
        for (int i = 0; i < n3; ++i) {
            SizeRequirements sizeRequirements = this.columnRequirements[n2 + i];
            n4 += (long)sizeRequirements.minimum;
            n5 += (long)sizeRequirements.preferred;
            n6 += (long)sizeRequirements.maximum;
        }
        int allocated = (int)view.getMinimumSpan(n);
        if ((long)allocated > n4) {
            SizeRequirements[] children = new SizeRequirements[n3];
            for (int j = 0; j < n3; ++j) {
                int n7 = j;
                SizeRequirements sizeRequirements2 = this.columnRequirements[n2 + j];
                children[n7] = sizeRequirements2;
                SizeRequirements sizeRequirements3 = sizeRequirements2;
                sizeRequirements3.maximum = Math.max(sizeRequirements3.maximum, (int)view.getMaximumSpan(n));
            }
            int[] spans = new int[n3];
            SizeRequirements.calculateTiledPositions(allocated, null, children, new int[n3], spans);
            for (int k = 0; k < n3; ++k) {
                SizeRequirements sizeRequirements4 = children[k];
                sizeRequirements4.minimum = Math.max(spans[k], sizeRequirements4.minimum);
                sizeRequirements4.preferred = Math.max(sizeRequirements4.minimum, sizeRequirements4.preferred);
                sizeRequirements4.maximum = Math.max(sizeRequirements4.preferred, sizeRequirements4.maximum);
            }
        }
        int allocated2 = (int)view.getPreferredSpan(n);
        if ((long)allocated2 > n5) {
            SizeRequirements[] children2 = new SizeRequirements[n3];
            for (int l = 0; l < n3; ++l) {
                children2[l] = this.columnRequirements[n2 + l];
            }
            int[] spans2 = new int[n3];
            SizeRequirements.calculateTiledPositions(allocated2, null, children2, new int[n3], spans2);
            for (int n8 = 0; n8 < n3; ++n8) {
                SizeRequirements sizeRequirements5 = children2[n8];
                sizeRequirements5.preferred = Math.max(spans2[n8], sizeRequirements5.preferred);
                sizeRequirements5.maximum = Math.max(sizeRequirements5.preferred, sizeRequirements5.maximum);
            }
        }
    }
    
    @Override
    protected View getViewAtPosition(int n, Rectangle rectangle) {
        int viewCount = this.getViewCount();
        int i = 0;
        while (i < viewCount) {
            View view = this.getView(i);
            int startOffset = view.getStartOffset();
            int endOffset = view.getEndOffset();
            if (n >= startOffset && n < endOffset) {
                if (rectangle != null) {
                    this.childAllocation(i, rectangle);
                    return view;
                }
                return view;
            }
            else {
                ++i;
            }
        }
        if (n == this.getEndOffset()) {
            View view2 = this.getView(viewCount - 1);
            if (rectangle != null) {
                this.childAllocation(viewCount - 1, rectangle);
                return view2;
            }
            return view2;
        }
        else {
            return null;
        }
    }
    
    static {
        EMPTY = new BitSet();
    }
    
    public class TableRow extends BoxView
    {
        BitSet fillColumns;
        int row;
        
        public TableRow(Element element) {
            super(element, 0);
            this.fillColumns = new BitSet();
        }
        
        void clearFilledColumns() {
            this.fillColumns.and(TableView.EMPTY);
        }
        
        void fillColumn(int bitIndex) {
            this.fillColumns.set(bitIndex);
        }
        
        boolean isFilled(int bitIndex) {
            return this.fillColumns.get(bitIndex);
        }
        
        int getRow() {
            return this.row;
        }
        
        void setRow(int row) {
            this.row = row;
        }
        
        int getColumnCount() {
            int n = 0;
            for (int size = this.fillColumns.size(), i = 0; i < size; ++i) {
                if (this.fillColumns.get(i)) {
                    ++n;
                }
            }
            return this.getViewCount() + n;
        }
        
        @Override
        public void replace(int n, int n2, View[] array) {
            super.replace(n, n2, array);
            TableView.this.invalidateGrid();
        }
        
        @Override
        protected void layoutMajorAxis(int n, int n2, int[] array, int[] array2) {
            for (int n3 = 0, viewCount = this.getViewCount(), i = 0; i < viewCount; ++i, ++n3) {
                View view = this.getView(i);
                while (this.isFilled(n3)) {
                    ++n3;
                }
                int columnsOccupied = TableView.this.getColumnsOccupied(view);
                array2[i] = TableView.this.columnSpans[n3];
                array[i] = TableView.this.columnOffsets[n3];
                if (columnsOccupied > 1) {
                    int length = TableView.this.columnSpans.length;
                    for (int j = 1; j < columnsOccupied; ++j) {
                        if (n3 + j < length) {
                            int n4 = i;
                            array2[n4] += TableView.this.columnSpans[n3 + j];
                        }
                    }
                    n3 += columnsOccupied - 1;
                }
            }
        }
        
        @Override
        protected void layoutMinorAxis(int n, int n2, int[] array, int[] array2) {
            super.layoutMinorAxis(n, n2, array, array2);
            for (int n3 = 0, viewCount = this.getViewCount(), i = 0; i < viewCount; ++i, ++n3) {
                View view = this.getView(i);
                while (this.isFilled(n3)) {
                    ++n3;
                }
                int columnsOccupied = TableView.this.getColumnsOccupied(view);
                int rowsOccupied = TableView.this.getRowsOccupied(view);
                if (rowsOccupied > 1) {
                    for (int j = 1; j < rowsOccupied; ++j) {
                        if (this.getRow() + j < TableView.this.getViewCount()) {
                            int span = TableView.this.getSpan(1, this.getRow() + j);
                            int n4 = i;
                            array2[n4] += span;
                        }
                    }
                }
                if (columnsOccupied > 1) {
                    n3 += columnsOccupied - 1;
                }
            }
        }
        
        @Override
        public int getResizeWeight(int n) {
            return 1;
        }
        
        @Override
        protected View getViewAtPosition(int n, Rectangle rectangle) {
            int viewCount = this.getViewCount();
            int i = 0;
            while (i < viewCount) {
                View view = this.getView(i);
                int startOffset = view.getStartOffset();
                int endOffset = view.getEndOffset();
                if (n >= startOffset && n < endOffset) {
                    if (rectangle != null) {
                        this.childAllocation(i, rectangle);
                        return view;
                    }
                    return view;
                }
                else {
                    ++i;
                }
            }
            if (n == this.getEndOffset()) {
                View view2 = this.getView(viewCount - 1);
                if (rectangle != null) {
                    this.childAllocation(viewCount - 1, rectangle);
                    return view2;
                }
                return view2;
            }
            else {
                return null;
            }
        }
    }
    
    @Deprecated
    public class TableCell extends BoxView implements TableView.GridCell
    {
        int row;
        int col;
        
        public TableCell(Element element) {
            super(element, 1);
        }
        
        @Override
        public int getColumnCount() {
            return 1;
        }
        
        @Override
        public int getRowCount() {
            return 1;
        }
        
        @Override
        public void setGridLocation(int row, int col) {
            this.row = row;
            this.col = col;
        }
        
        @Override
        public int getGridRow() {
            return this.row;
        }
        
        @Override
        public int getGridColumn() {
            return this.col;
        }
    }
    
    interface GridCell
    {
        public abstract void setGridLocation(int p0, int p1);
        
        public abstract int getGridRow();
        
        public abstract int getGridColumn();
        
        public abstract int getColumnCount();
        
        public abstract int getRowCount();
    }
}
